/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2004 Lincoln Phipps <lincoln.phipps@openmutual.net>
 *
 * Blatantly copied from planner-task-dialog.c with 
 * function names changed and cludged so thanks goes to.....
 * Richard Hult, Mikael Hallendal and Alvaro del Castillo.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <time.h>
#include <string.h>
#include <stdlib.h> /* for atoi */
#include <libgnome/gnome-i18n.h>
#include <glade/glade.h>
#include <gtk/gtk.h>
#include <libplanner/mrp-object.h>
#include <libplanner/mrp-project.h>
#include "planner-cell-renderer-list.h"
#include "planner-assignment-model.h"
#include "planner-predecessor-model.h"
#include "planner-task-use-resource-dialog.h"

typedef struct {
	PlannerWindow  *main_window;
	MrpTask       *task;
	GtkWidget     *dialog;
	GtkWidget     *resource_list;
	GtkWidget     *name_entry;
} DialogData;


static void  task_use_resource_dialog_close_clicked_cb           (GtkWidget           *w, 
						     DialogData          *data);
static void  task_use_resource_dialog_assignment_toggled_cb      (GtkCellRendererText *cell,
						     gchar               *path_str,
						     DialogData          *data);
static void  task_use_resource_dialog_setup_resource_list        (DialogData          *data); 
static void  task_use_resource_dialog_resource_units_cell_edited (GtkCellRendererText *cell, 
						     gchar               *path_str,
						     gchar               *new_text, 
						     DialogData          *data);
static void  task_use_resource_dialog_update_title               (DialogData           *data);


/* Keep the dialogs here so that we can just raise the dialog if it's
 * opened twice for the same task.
 */
static GHashTable *dialogs = NULL;

#define DIALOG_GET_DATA(d) g_object_get_data ((GObject*)d, "data")


static void
task_use_resource_dialog_close_clicked_cb (GtkWidget *w, DialogData *data)
{
	gtk_widget_destroy (data->dialog);
}


static void  
task_use_resource_dialog_resource_units_cell_edited (GtkCellRendererText *cell, 
					gchar               *path_str,
					gchar               *new_text, 
					DialogData          *data)
{
	GtkTreeView   *tree;
	GtkTreeModel  *model;
	GtkTreePath   *path;
	GtkTreeIter    iter;
	MrpResource   *resource;
	MrpAssignment *assignment;
	
	tree = GTK_TREE_VIEW (data->resource_list);
	
	model = gtk_tree_view_get_model (tree);

	path = gtk_tree_path_new_from_string (path_str);
	gtk_tree_model_get_iter (model, &iter, path);
	gtk_tree_path_free (path);

	resource = ((GList *)iter.user_data)->data;
		
	assignment = mrp_task_get_assignment (data->task, resource);

	if (assignment) {
		g_object_set (assignment,
			      "units", atoi (new_text),
			      NULL);
	}
}



static void
task_use_resource_dialog_assignment_toggled_cb (GtkCellRendererText *cell,
				   gchar               *path_str,
				   DialogData          *data)
{
	GtkTreeView  *tree;
	GtkTreeModel *model;
	GtkTreePath  *path;
	GtkTreeIter   iter;
	MrpResource  *resource;
	gboolean      active;
	gboolean      inconsistent;

	tree = GTK_TREE_VIEW (data->resource_list);
	
	g_object_get (cell, "active", &active, NULL);
	g_object_get (cell, "inconsistent", &inconsistent, NULL);

	model = gtk_tree_view_get_model (tree);

	path = gtk_tree_path_new_from_string (path_str);
	
	gtk_tree_model_get_iter (model, &iter, path);

	gtk_tree_path_free (path);

	resource = ((GList *)iter.user_data)->data;

 	if (!active) {
		mrp_resource_assign (resource, data->task, 100);
	} else {
		MrpAssignment *assignment;
		
		assignment = mrp_task_get_assignment (data->task, resource);

		if (assignment) {
			mrp_object_removed (MRP_OBJECT (assignment));
		} 
	}
}


static void
task_use_resource_dialog_setup_resource_list (DialogData *data)
{
	GtkTreeView       *tree = GTK_TREE_VIEW (data->resource_list);
	GtkTreeViewColumn *col;
	GtkCellRenderer   *cell;
	GtkTreeModel      *model;

	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (tree),
				     GTK_SELECTION_SINGLE);
	
	/* Name */
	cell = gtk_cell_renderer_text_new ();
	g_object_set (cell, "editable", FALSE, NULL);
	
	col = gtk_tree_view_column_new_with_attributes (_("Name"),
							cell,
							"text", RESOURCE_ASSIGNMENT_COL_NAME,
							NULL);
	gtk_tree_view_column_set_resizable (col, TRUE);
	gtk_tree_view_column_set_min_width (col, 200);
	gtk_tree_view_append_column (tree, col);
	
	/* Assigned. */
	cell = gtk_cell_renderer_toggle_new ();
	g_object_set (cell,
		      "activatable", TRUE,
		      NULL);
		      
	g_signal_connect (cell,
			  "toggled",
			  G_CALLBACK (task_use_resource_dialog_assignment_toggled_cb),
			  data);
	
	col = gtk_tree_view_column_new_with_attributes (_("Assigned"),
							cell,
							"active", RESOURCE_ASSIGNMENT_COL_ASSIGNED,
							NULL);

	gtk_tree_view_column_set_resizable (col, TRUE);
	gtk_tree_view_append_column (tree, col);

	/* Assigned units */
	cell = gtk_cell_renderer_text_new ();
	
	g_signal_connect (cell,
			  "edited",
			  G_CALLBACK (task_use_resource_dialog_resource_units_cell_edited),
			  data);
	
	col = gtk_tree_view_column_new_with_attributes (_("Units"),
						cell,
						"text", RESOURCE_ASSIGNMENT_COL_ASSIGNED_UNITS,
						"editable", RESOURCE_ASSIGNMENT_COL_ASSIGNED_UNITS_EDITABLE,
						NULL);
	gtk_tree_view_column_set_resizable (col, TRUE);
	gtk_tree_view_column_set_min_width (col, 50);
	gtk_tree_view_append_column (tree, col);

	model = GTK_TREE_MODEL (planner_assignment_model_new (data->task));
	
	gtk_tree_view_set_model (tree, model);
}

static void
task_use_resource_dialog_setup_widgets (DialogData *data,
			   GladeXML   *glade)
{
	GtkWidget    *w;

	w = glade_xml_get_widget (glade, "close_button");
	g_signal_connect (w,
			  "clicked",
			  G_CALLBACK (task_use_resource_dialog_close_clicked_cb),
			  data);
}




static void
task_use_resource_dialog_update_title (DialogData *data)
{
	gchar *title;
	gchar *name;

	g_object_get (data->task, "name", &name, NULL);

	if (!name || strlen (name) == 0) {
		title = g_strdup (_("Use Resources"));
	} else {
		title = g_strconcat (name, " - ", _("Use Resources"), NULL);
	}

	gtk_window_set_title (GTK_WINDOW (data->dialog), title);

	g_free (name);
	g_free (title);
}	

static void
task_use_resource_dialog_parent_destroy_cb (GtkWidget *parent,
			       GtkWidget *dialog)
{
	gtk_widget_destroy (dialog);
}

static void
task_use_resource_dialog_destroy_cb (GtkWidget  *parent,
			DialogData *data)
{
	g_hash_table_remove (dialogs, data->task);
}

/* 
*  This gets passed a list of tasks which could be 1 or more tasks 
*  It opens 1 dialog per task. If someone selects 10 tasks then 
*  10 dialog boxes are opened at once (layered on top of each other).
*/

GtkWidget *
planner_task_use_resource_dialog_new (PlannerWindow *window,
		    GList      *list)
{
	GList	*l;
	MrpTask      *task;
	DialogData   *data;
	GladeXML     *glade;
	GtkWidget    *dialog = NULL;
	
	for (l = list; l; l = l->next) {
		task = l->data;
		g_return_val_if_fail (MRP_IS_TASK (task), NULL);
		
		if (!dialogs) {
			dialogs = g_hash_table_new (NULL, NULL);
		}	
	
		dialog = g_hash_table_lookup (dialogs, task);
		if (dialog) {
			gtk_window_present (GTK_WINDOW (dialog));
			return dialog;
		}

		glade = glade_xml_new (GLADEDIR "/task-use-resource-dialog.glade",
			       NULL,
			       GETTEXT_PACKAGE);
		if (!glade) {
			g_warning ("Could not create task dialog.");
			return NULL;
		}

		dialog = glade_xml_get_widget (glade, "task_use_resource");
	
		data = g_new0 (DialogData, 1);

		data->main_window = window;
		data->dialog = dialog;
		data->task = task;

		g_hash_table_insert (dialogs, task, dialog);

		g_signal_connect (dialog,
			  	"destroy",
			  	G_CALLBACK (task_use_resource_dialog_destroy_cb),
			  	data);
	
		g_signal_connect_object (window,
				 "destroy",
				 G_CALLBACK (task_use_resource_dialog_parent_destroy_cb),
				 dialog,
				 0);

		data->resource_list = glade_xml_get_widget (glade, "resource_list");
		
		task_use_resource_dialog_setup_resource_list (data);

		g_object_set_data_full (G_OBJECT (dialog),
				"data", data,
				g_free);
	
		task_use_resource_dialog_setup_widgets (data, glade);

		task_use_resource_dialog_update_title (data);
		}
	return dialog;
}





